package ink.wgink.module.instantmessage.websocket.handler.text;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import ink.wgink.module.instantmessage.websocket.enums.ClientSocketTypeEnum;
import ink.wgink.module.instantmessage.websocket.enums.SendStatusEnum;
import ink.wgink.module.instantmessage.websocket.exception.*;
import ink.wgink.module.instantmessage.websocket.manager.WebSocketChannelManager;
import ink.wgink.module.instantmessage.websocket.pojo.WebSocketClientMessage;
import ink.wgink.module.instantmessage.websocket.pojo.WebSocketSession;
import ink.wgink.module.instantmessage.websocket.pojo.body.RegisterBody;
import ink.wgink.module.instantmessage.websocket.pojo.body.SendStatusBody;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import org.apache.commons.compress.utils.Sets;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * @ClassName: WebSocketTextHandler
 * @Description: WebSocket文本处理器
 * @Author: wanggeng
 * @Date: 2021/9/12 8:08 下午
 * @Version: 1.0
 */
public class WebSocketTextHandler {

    public void handler(ChannelHandlerContext ctx, TextWebSocketFrame textWebSocketFrame) {
        WebSocketClientMessage clientSocketMessage = null;
        SendStatusBody sendStatusBody = null;
        try {
            clientSocketMessage = JSONObject.parseObject(textWebSocketFrame.text(), WebSocketClientMessage.class);
            if (clientSocketMessage.getType() == null) {
                throw new TypeErrorException("Type类型不能为空");
            }
            if (StringUtils.isBlank(clientSocketMessage.getBody())) {
                throw new BodyException("Body主体不能为空");
            }
            if (ClientSocketTypeEnum.REGISTER.getValue() == clientSocketMessage.getType()) {
                // 客户端注册消息
                sendStatusBody = clientRegisterSession(ctx.channel(), clientSocketMessage);
            } else if (ClientSocketTypeEnum.MESSAGE.getValue() == clientSocketMessage.getType()) {
                // 文本消息
                sendText(ctx.channel(), clientSocketMessage);
            } else if (ClientSocketTypeEnum.GROUP_MESSAGE.getValue() == clientSocketMessage.getType()) {
                // 群发消息
                sendGroupText(ctx.channel(), clientSocketMessage);
            } else if (ClientSocketTypeEnum.SEARCH_ONLINE_USER.getValue() == clientSocketMessage.getType()) {
                // 用户在线状态
                listOnlineUser(ctx.channel(), clientSocketMessage);
            }
        } catch (SessionException e) {
            // 没有登录时，返回内容，关闭连接
            sendStatusBody = new SendStatusBody(SendStatusEnum.SESSION_ERROR.getValue(), SendStatusEnum.SESSION_ERROR, e.getMessage());
            clientSocketMessage.setBody(JSONObject.toJSONString(sendStatusBody));
            WebSocketChannelManager.getInstance().sendText(ctx.channel(), clientSocketMessage);
            ctx.close();
        } catch (TypeErrorException e) {
            sendStatusBody = new SendStatusBody(SendStatusEnum.SESSION_ERROR.getValue(), SendStatusEnum.SESSION_ERROR, e.getMessage());
        } catch (BodyException e) {
            sendStatusBody = new SendStatusBody(SendStatusEnum.SESSION_ERROR.getValue(), SendStatusEnum.SESSION_ERROR, e.getMessage());
        } catch (UserErrorException e) {
            sendStatusBody = new SendStatusBody(SendStatusEnum.SESSION_ERROR.getValue(), SendStatusEnum.SESSION_ERROR, e.getMessage());
        } catch (JSONException e) {
            clientSocketMessage = new WebSocketClientMessage();
            clientSocketMessage.setSystem(true);
            clientSocketMessage.setFrom(WebSocketChannelManager.FORM_SYSTEM);
            sendStatusBody = new SendStatusBody(SendStatusEnum.TO_ERROR.getValue(), SendStatusEnum.MESSAGE_ERROR, e.getMessage());
        } finally {
            if (sendStatusBody != null && ctx.channel().isOpen()) {
                clientSocketMessage.setBody(JSONObject.toJSONString(sendStatusBody));
                WebSocketChannelManager.getInstance().sendText(ctx.channel(), clientSocketMessage);
            }
        }
    }

    /**
     * 客户端会话注册
     *
     * @param channel             会话通道
     * @param clientSocketMessage 客户端信息
     * @return
     * @throws SessionException
     */
    private SendStatusBody clientRegisterSession(Channel channel, WebSocketClientMessage clientSocketMessage) throws SessionException {
        RegisterBody registerBody = JSONObject.parseObject(clientSocketMessage.getBody(), RegisterBody.class);
        // 更新通道
        WebSocketChannelManager.getInstance().addChannel(registerBody.getSessionId(), clientSocketMessage.getFrom(), channel);
        return new SendStatusBody(SendStatusEnum.SUCCESS.getValue(), SendStatusEnum.SUCCESS, "OK");
    }

    /**
     * 发送文字消息
     *
     * @param channel                通道
     * @param webSocketClientMessage 客户端消息
     * @throws UserErrorException
     */
    private void sendText(Channel channel, WebSocketClientMessage webSocketClientMessage) throws UserErrorException, UserSessionException {
        if (StringUtils.isBlank(webSocketClientMessage.getTo())) {
            throw new UserErrorException("To 值不能为空");
        }
        webSocketClientMessage.setSystem(false);
        List<WebSocketSession> webSocketSessions = WebSocketChannelManager.getInstance().listOnlineUser(webSocketClientMessage.getTo());
        // 返回失败结果
        if (webSocketSessions.isEmpty()) {
            sendTextFailed(channel, webSocketClientMessage, "用户不在线");
            return;
        }
        // 发送消息
        for (WebSocketSession webSocketSession : webSocketSessions) {
            WebSocketChannelManager.getInstance().sendText(webSocketSession.getChannel(), webSocketClientMessage);
        }
    }

    /**
     * 群发文字消息
     *
     * @param channel                通道
     * @param webSocketClientMessage 客户端消息
     * @throws UserErrorException
     * @throws UserSessionException
     */
    private void sendGroupText(Channel channel, WebSocketClientMessage webSocketClientMessage) throws UserErrorException, UserSessionException {
        if (StringUtils.isBlank(webSocketClientMessage.getTo())) {
            throw new UserErrorException("To 值不能为空");
        }
        webSocketClientMessage.setSystem(false);
        List<String> toUserIds = new ArrayList<>(Sets.newHashSet(webSocketClientMessage.getTo().split(",")));
        List<WebSocketSession> webSocketSessions = WebSocketChannelManager.getInstance().listOnlineUser(toUserIds);
        // 返回失败结果
        if (webSocketSessions.isEmpty()) {
            sendTextFailed(channel, webSocketClientMessage, "无用户在线");
            return;
        }
        // 发送消息
        for (WebSocketSession webSocketSession : webSocketSessions) {
            WebSocketChannelManager.getInstance().sendText(webSocketSession.getChannel(), webSocketClientMessage);
        }
    }

    private void listOnlineUser(Channel channel, WebSocketClientMessage clientSocketMessage) {

    }

    /**
     * 发送文本失败
     *
     * @param fromChannel            来源通道
     * @param webSocketClientMessage webSocket客户端消息
     * @param msg
     */
    public static void sendTextFailed(Channel fromChannel, WebSocketClientMessage webSocketClientMessage, String msg) {
        webSocketClientMessage.setType(ClientSocketTypeEnum.SEND_STATUS.getValue());
        webSocketClientMessage.setBody(JSONObject.toJSONString(new SendStatusBody(SendStatusEnum.FAILED.getValue(), SendStatusEnum.FAILED, msg)));
        WebSocketChannelManager.getInstance().sendText(fromChannel, webSocketClientMessage);
    }

}
